import React, { useState, useContext, useEffect, useRef } from 'react';
import { AppContext } from './../utils/Context';
import { URLParamsPopulator } from './../utils';
import Slider from 'rc-slider';
import { AnimatePresence, motion } from 'framer-motion';

const RadioGroup = ({ optionName, optionSlug, options, filtersReset }) => {
	const { frontEndFilters, setFrontEndFilters, colors } = useContext(AppContext);

	if (!options) {
		return '';
	}

	const handleChange = (e) => {
		const { value } = e.target;

		let newRadioValue = frontEndFilters[optionSlug] === value ? '' : value;

		if( filtersReset && newRadioValue !== '' ){
			setFrontEndFilters({
				[optionSlug]: newRadioValue
			});

			URLParamsPopulator({
				[optionSlug]: newRadioValue
			});
		} else {
			setFrontEndFilters({
				...frontEndFilters,
				[optionSlug]: newRadioValue
			});

			URLParamsPopulator({
				...frontEndFilters,
				[optionSlug]: newRadioValue
			});
		}
	};

	const groupClassName = "linear-listings__filters__group linear-listings__filters__group__" + optionSlug;

	return (
		<>
			<motion.div
				animate={{ opacity: 1, y: 0 }}
				initial={{ opacity: 0, y: 20 }}
				exit={{ opacity: 0, y: 20 }}
				transition={{
					duration: 0.5,
					type: 'spring',
				}}
				className={groupClassName}
			>
				{options.map((option, index) => {
					const checked = typeof frontEndFilters[optionSlug] !== 'undefined' ? ( frontEndFilters[optionSlug] === option.value ? true : false ) : false;
					const filterClass = "linear-listings__filters__checkbox " + "linear-listings__filters__" + optionSlug + "__" + option.value;

					return (
						<div
							key={optionName + '_' + index}
							className={filterClass}
						>
							<input
								type="checkbox"
								id={optionSlug + '_' + index}
								name={optionName}
								value={option.value}
								checked={checked}
								onChange={(e) => handleChange(e, index)}
							/>
							<label htmlFor={optionSlug + '_' + index}>
								{option.label}
							</label>
						</div>
					);
				})}
			</motion.div>
		</>
	);
};

const SelectGroup = ({ optionName, optionSlug, options }) => {
	const { frontEndFilters, setFrontEndFilters, colors } = useContext(AppContext);

	if (!options) {
		return '';
	}

	const handleChange = (e, index, options) => {
		const { checked } = e.target;

		// initial setup if necessary
		let currentStateCheckboxes =
			typeof frontEndFilters[optionSlug] !== 'undefined'
				? frontEndFilters[optionSlug]
				: new Array(options.length).fill(false);

		// flip value
		currentStateCheckboxes[index] = checked;

		setFrontEndFilters({
			...frontEndFilters,
			[optionSlug]: currentStateCheckboxes,
		});

		URLParamsPopulator({
			...frontEndFilters,
			[optionSlug]: currentStateCheckboxes,
		});
	};

	const groupClassName = "linear-listings__filters__group linear-listings__filters__group__" + optionSlug;

	return (
		<>
			<motion.div
				animate={{ opacity: 1, y: 0 }}
				initial={{ opacity: 0, y: 20 }}
				exit={{ opacity: 0, y: 20 }}
				transition={{
					duration: 0.5,
					type: 'spring',
				}}
				className={groupClassName}
				data-filters-count={options.length}
			>
				{options.map((option, index) => {
					const checked = typeof frontEndFilters[optionSlug] !== 'undefined' ? frontEndFilters[optionSlug][index] : false;
					const filterClass = "linear-listings__filters__checkbox " + "linear-listings__filters__" + optionSlug + "__" + option.value;

					return (
						<div
							key={optionName + '_' + index}
							className={filterClass}
						>
							<input
								type="checkbox"
								id={optionSlug + '_' + index}
								name={optionName}
								value={option.value}
								onChange={(e) =>
									handleChange(e, index, options)
								}
								checked={checked}
							/>
							<label htmlFor={optionSlug + '_' + index}>
								{option.label}
							</label>
						</div>
					);
				})}
			</motion.div>
		</>
	);
};

const ProductGroup = () => {
	const { texts } = useContext(AppContext);

	return (
		<>
			<RadioGroup
				optionName={texts.apartments}
				optionSlug="productGroup"
				options={[
					{ value: 'apartments', label: texts.apartments },
					{ value: 'plots', label: texts.plots, reset: true },
					{ value: 'farms', label: texts.farms, reset: true },
					{ value: 'garages', label: texts.garages, reset: true },
					{
						value: 'vacation_apartment',
						label: texts.vacationApartment,
					},
				]}
				filtersReset={true}
			/>
		</>
	);
};

const RoomCount = () => {
	const { texts } = useContext(AppContext);

	return (
		<>
			<SelectGroup
				optionName={texts.roomCount}
				optionSlug="roomCount"
				options={[
					{ value: '1', label: '1 ' + texts.room },
					{ value: '2', label: '2 ' + texts.rooms },
					{ value: '3', label: '3 ' + texts.rooms },
					{ value: '4', label: '4 ' + texts.rooms },
					{ value: '5', label: '5 ' + texts.rooms },
					{ value: '6', label: '5+ ' + texts.rooms },
				]}
			/>
		</>
	);
};

const ListingType = ({ frontEndFilters, setFrontEndFilters }) => {
	const { texts } = useContext(AppContext);

	return (
		<>
			<RadioGroup
				optionName={texts.listingType}
				optionSlug="listingType"
				options={[
					{ value: 'flat', label: texts.flat },
					{ value: 'rowhouse', label: texts.rowhouse },
					{ value: 'pairhouse', label: texts.pairhouse },
					{ value: 'detachedhouse', label: texts.detachedHouse },
				]}
				frontEndFilters={frontEndFilters}
				setFrontEndFilters={setFrontEndFilters}
			/>
		</>
	);
};

const CommissionType = ({ frontEndFilters, setFrontEndFilters }) => {
	const { texts } = useContext(AppContext);

	return (
		<>
			<RadioGroup
				optionName={texts.commissionType}
				optionSlug="commissionType"
				options={[
					{ value: 'sell', label: texts.salesCommissions, reset: true },
					{ value: 'rent', label: texts.rentCommissions, reset: true }
				]}
				filtersReset={true}
			/>
		</>
	);
};

const Specifications = () => {
	const { frontEndFilters, setFrontEndFilters, texts } = useContext(AppContext);

	return (
		<>
			<SelectGroup
				optionName={texts.sauna}
				optionSlug="sauna"
				options={[{ value: 'sauna', label: texts.sauna }]}
				frontEndFilters={frontEndFilters}
				setFrontEndFilters={setFrontEndFilters}
			/>
			<SelectGroup
				optionName={texts.balcony}
				optionSlug="hasBalcony"
				options={[{ value: 'balcony', label: texts.balcony }]}
				frontEndFilters={frontEndFilters}
				setFrontEndFilters={setFrontEndFilters}
			/>
			<SelectGroup
				optionName={texts.elevator}
				optionSlug="housingCooperativeElevator"
				options={[{ value: 'elevator', label: texts.elevator }]}
				frontEndFilters={frontEndFilters}
				setFrontEndFilters={setFrontEndFilters}
			/>
		</>
	);
};

const PriceRange = () => {
	const priceMin = 15000;
	const priceMinStyled = priceMin.toString().replace(/\B(?=(\d{3})+(?!\d))/g, " ");
	const priceMax = 750000;
	const priceMaxStyled = priceMax.toString().replace(/\B(?=(\d{3})+(?!\d))/g, " ");
	const { frontEndFilters, setFrontEndFilters, colors, texts } = useContext(AppContext);
	const defaultValueLow = typeof frontEndFilters['priceMin'] !== 'undefined' ? frontEndFilters['priceMin'] : priceMinStyled;
	const defaultValueHigh = typeof frontEndFilters['priceMax'] !== 'undefined' ? frontEndFilters['priceMax'] : priceMaxStyled;
	const [priceRange, setPriceRange] = useState([defaultValueLow, defaultValueHigh]);
	const [onLoad, setOnLoad] = useState(true);

	const handleInputChange = (e, index) => {
		const { value } = e.target;

		let newState = [...priceRange];
		newState[index] = value;
		
		setPriceRange(newState);
		setFrontEndFilters({
			...frontEndFilters,
			['priceMin']: parseInt(newState[0].toString().replace(" ","")),
			['priceMax']: parseInt(newState[1].toString().replace(" ","")),
		});

		URLParamsPopulator({
			...frontEndFilters,
			['priceMin']: parseInt(newState[0].toString().replace(" ","")),
			['priceMax']: parseInt(newState[1].toString().replace(" ","")),
		});

		setOnLoad( false );
	};

	const handleRangeChange = (value) => {
		setPriceRange( value )
	}

	useEffect(() => {
		if(
			onLoad && 
			parseInt(priceRange[0].toString().replace(' ', '')) <= priceMin && 
			parseInt(priceRange[1].toString().replace(' ', '')) === priceMax
		){
			// init, dont' set values to filters
		} else {
			setFrontEndFilters({
				...frontEndFilters,
				['priceMin']: parseInt(priceRange[0].toString().replace(' ', '')),
				['priceMax']: parseInt(priceRange[1].toString().replace(' ', '')),
			});
		}

		URLParamsPopulator({
			...frontEndFilters,
			['priceMin']: parseInt(priceRange[0].toString().replace(' ', '')),
			['priceMax']: parseInt(priceRange[1].toString().replace(' ', '')),
		});

		setOnLoad( false );

	}, [priceRange]);

	// ensure no nulls
	if( !priceRange[0] ){
		priceRange[0] = 0;
	}
	if( !priceRange[1] ){
		priceRange[1] = 0;
	}

	// Style numbers
	priceRange[0] = priceRange[0].toString().replace(' ', '').replace(/\B(?=(\d{3})+(?!\d))/g, " ");
	priceRange[1] = priceRange[1].toString().replace(' ', '').replace(/\B(?=(\d{3})+(?!\d))/g, " ");

	// Min-value indicator
	if (parseInt(priceRange[0].toString().replace(' ', '')) === parseInt( priceMin )) {
		priceRange[0] = 0;
	}

	// Max-value indicator
	if (parseInt(priceRange[1].toString().replace(' ', '')) === parseInt( priceMax )) {
		if( !priceRange[1].includes("+") ){
			priceRange[1] = priceRange[1] + '+';
		}
	}

	return (
		<>
			<motion.div
				animate={{ opacity: 1, y: 0 }}
				initial={{ opacity: 0, y: 20 }}
				exit={{ opacity: 0, y: 20 }}
				transition={{
					duration: 0.5,
					type: 'spring',
				}}
				className="linear-listings__filters__group linear-listings__filters__group__priceRange linear-listings__filters__group--wide"
			>
				<div className="linear-listings__filters__range">
					<span className="linear-listings__filters__range__input">
						<input
							id="linearPriceMin"
							type="text"
							value={priceRange[0]}
							onChange={(e) => handleInputChange(e, 0)}
						/>
						<label htmlFor="linearPriceMin">
							{texts.setFilterMinimumPrice}
						</label>
						<span className="linear-listings__filters__range__input__euro">€</span>
					</span>
					<Slider
						range
						min={priceMin}
						max={priceMax}
						step={5000}
						onChange={(value) => handleRangeChange(value)}
						value={[
							parseInt( priceRange[0].toString().replace(" ","") ),
							parseInt( priceRange[1].toString().replace(" ","") ),
						]}
						className="linear-listings__filters__range__slider"
						trackStyle={{ backgroundColor: colors.primary_color }}
						railStyle={{ backgroundColor: '#d6d8da' }}
						pushable={true}
						handleStyle={{
							borderColor: colors.primary_color,
							backgroundColor: 'white',
						}}
					/>
					<span className="linear-listings__filters__range__input">
						<input
							id="linearPriceMax"
							type="text"
							value={priceRange[1]}
							onChange={(e) => handleInputChange(e, 1)}
						/>
						<label htmlFor="linearPriceMax">
							{texts.setFilterMaximumPrice}
						</label>
						<span className="linear-listings__filters__range__input__euro">€</span>
					</span>
				</div>
			</motion.div>
		</>
	);
};

const RentRange = () => {
	const rentMin = 0;
	const rentMinStyled = rentMin.toString().replace(/\B(?=(\d{3})+(?!\d))/g, " ");
	const rentMax = 1500;
	const rentMaxStyled = rentMax.toString().replace(/\B(?=(\d{3})+(?!\d))/g, " ");
	const { frontEndFilters, setFrontEndFilters, colors, texts } = useContext(AppContext);
	const defaultValueLow = typeof frontEndFilters['rentMin'] !== 'undefined' ? frontEndFilters['rentMin'] : rentMinStyled;
	const defaultValueHigh = typeof frontEndFilters['rentMax'] !== 'undefined' ? frontEndFilters['rentMax'] : rentMaxStyled;
	const [rentRange, setRentRange] = useState([defaultValueLow, defaultValueHigh]);
	const [onLoad, setOnLoad] = useState(true);

	const handleInputChange = (e, index) => {
		const { value } = e.target;

		let newState = [...rentRange];
		newState[index] = value;

		setRentRange(newState);
		setFrontEndFilters({
			...frontEndFilters,
			['rentMin']: parseInt(newState[0].toString().replace(" ","")),
			['rentMax']: parseInt(newState[1].toString().replace(" ","")),
		});

		URLParamsPopulator({
			...frontEndFilters,
			['rentMin']: parseInt(newState[0].toString().replace(" ","")),
			['rentMax']: parseInt(newState[1].toString().replace(" ","")),
		});

		setOnLoad( false );
	};

	const handleRangeChange = (value) => {
		setRentRange( value );
	}

	useEffect(() => {
		if(
			onLoad && 
			parseInt(rentRange[0].toString().replace(' ', '')) <= rentMin && 
			parseInt(rentRange[1].toString().replace(' ', '')) === rentMax
		){
			// init, dont' set values to filters
		} else {
			setFrontEndFilters({
				...frontEndFilters,
				['rentMin']: parseInt(rentRange[0].toString().replace(" ","")),
				['rentMax']: parseInt(rentRange[1].toString().replace(" ","")),
			});
		}

		URLParamsPopulator({
			...frontEndFilters,
			['rentMin']: parseInt(rentRange[0].toString().replace(" ","")),
			['rentMax']: parseInt(rentRange[1].toString().replace(" ","")),
		});

		setOnLoad( false );
	}, [rentRange]);

	// ensure no nulls
	if( !rentRange[0] ){
		rentRange[0] = 0;
	}
	if( !rentRange[1] ){
		rentRange[1] = 0;
	}

	// Style numbers
	rentRange[0] = rentRange[0].toString().replace(' ', '').replace(/\B(?=(\d{3})+(?!\d))/g, " ");
	rentRange[1] = rentRange[1].toString().replace(' ', '').replace(/\B(?=(\d{3})+(?!\d))/g, " ");

	// Min-value indicator
	if (parseInt(rentRange[0].toString().replace(' ', '')) === parseInt( rentMin )) {
		rentRange[0] = 0;
	}

	// Max-value indicator
	if (parseInt(rentRange[1].toString().replace(' ', '')) === parseInt( rentMax )) {
		if( !rentRange[1].includes("+") ){
			rentRange[1] = rentRange[1] + ' +';
		}
	}

	return (
		<>
			<motion.div
				animate={{ opacity: 1, y: 0 }}
				initial={{ opacity: 0, y: 20 }}
				exit={{ opacity: 0, y: 20 }}
				transition={{
					duration: 0.5,
					type: 'spring',
				}}
				className="linear-listings__filters__group linear-listings__filters__group__rentRange linear-listings__filters__group--wide"
			>
				<div className="linear-listings__filters__range">
					<span className="linear-listings__filters__range__input">
						<input
							id="linearRentMin"
							type="text"
							value={rentRange[0]}
							onChange={(e) => handleInputChange(e, 0)}
						/>
						<label htmlFor="linearRentMin">
							{texts.setFilterMinimumRent}
						</label>
						<span className="linear-listings__filters__range__input__euro">€</span>
					</span>
					<Slider
						range
						min={rentMin}
						max={rentMax}
						step={25}
						onChange={(value) => handleRangeChange(value)}
						value={[
							parseInt( rentRange[0].toString().replace(" ","") ),
							parseInt( rentRange[1].toString().replace(" ","") ),
						]}
						className="linear-listings__filters__range__slider"
						trackStyle={{ backgroundColor: colors.primary_color }}
						railStyle={{ backgroundColor: '#d6d8da' }}
						pushable={true}
						handleStyle={{
							borderColor: colors.primary_color,
							backgroundColor: 'white',
						}}
					/>
					<span className="linear-listings__filters__range__input">
						<input
							id="linearRentMax"
							type="text"
							value={rentRange[1]}
							onChange={(e) => handleInputChange(e, 0)}
						/>
						<label htmlFor="linearRentMax">
							{texts.setFilterMaximumRent}
						</label>
						<span className="linear-listings__filters__range__input__euro">€</span>
					</span>
				</div>
			</motion.div>
		</>
	);
};

const SearchOptions = ({showSearch}) => {
	const { frontEndFilters, setFrontEndFilters, texts, searchOptions } = useContext(AppContext);

	const handleChange = (e) => {
		const { value } = e.target;

		let newValue = frontEndFilters['searchOption'] === value ? '' : value;

		setFrontEndFilters({
			...frontEndFilters,
			['searchOption']: newValue
		});

		URLParamsPopulator({
			...frontEndFilters,
			['searchOption']: newValue
		});
	};

	if( 
		searchOptions.length === 0 || 
		searchOptions === '' || 
		JSON.stringify(searchOptions) === JSON.stringify([''])
	){
		// Return classic search instead
		if( !showSearch ){
			return "";
		}

		return <Search />;
	}

	return (
		<>
			<motion.div
				animate={{ opacity: 1, y: 0 }}
				initial={{ opacity: 0, y: 20 }}
				exit={{ opacity: 0, y: 20 }}
				transition={{
					duration: 0.5,
					type: 'spring',
				}}
				className="linear-listings__filters__group linear-listings__filters__group__searchOptions"
			>
				<div className="linear-listings__filters__select">
					<select value={frontEndFilters.searchOption} onChange={(e) => handleChange(e)}>
						<option key={'empty'} value={""}>{texts.chooseLocation}</option>
						{searchOptions.map((option, i) => {         
							return ( <option key={i} value={option}>{option}</option> );
						})}
					</select>
				</div>
			</motion.div>
		</>
	);
}

const Search = () => {
	const { frontEndFilters, setFrontEndFilters, texts } = useContext(AppContext);

	const handleChange = (e) => {
		const { value } = e.target;

		let newValue = frontEndFilters['search'] === value ? '' : value;

		setFrontEndFilters({
			...frontEndFilters,
			['search']: newValue
		});

		URLParamsPopulator({
			...frontEndFilters,
			['search']: newValue
		});
	};

	return (
		<>
			<motion.div
				animate={{ opacity: 1, y: 0 }}
				initial={{ opacity: 0, y: 20 }}
				exit={{ opacity: 0, y: 20 }}
				transition={{
					duration: 0.5,
					type: 'spring',
				}}
				className="linear-listings__filters__group linear-listings__filters__group__search"
			>
				<div className="linear-listings__filters__input">
					<input 
						type="text"
						name={texts.search}
						onChange={handleChange}
						value={frontEndFilters.search}
						placeholder={texts.search}
					/>
				</div>
			</motion.div>
		</>
	);
}

const BusinessListingType = () => {
	const { texts } = useContext(AppContext);

	return (
		<>
			<RadioGroup
				optionName={texts.businessListingType}
				optionSlug="businessListingType"
				options={[
					{ value: 'office_space', label: texts.officeSpace },
					{ value: 'business_space', label: texts.businessSpace },
					{ value: 'production_space', label: texts.productionSpace },
					{ value: 'storage_space', label: texts.storageSpaceHouse },
				]}
			/>
		</>
	);
};

const FiltersList = ({ showRangeSliders, showSearch }) => {
	const { listingsType, frontEndFilters } = useContext(AppContext);
	let showExtendedFilters = false;

	// Check if we want to show extended filters
	if( typeof frontEndFilters.productGroup !== 'undefined' ){
		if( 
			['apartments', 'vacation_apartment'].includes( frontEndFilters.productGroup ) ||
			frontEndFilters.productGroup === ""
		){
			showExtendedFilters = true;
		}
	} else {
		showExtendedFilters = true;
	}

	// Check if we have internal selections that affect the filters shown
	let showBusinessListingType = true;
	if( typeof frontEndFilters.commissionType !== 'undefined' ){
		if( frontEndFilters.commissionType === 'sell' ){
			showBusinessListingType = false;
		}
	}

	// easy return everything
	if (listingsType === 'all') {
		return (
			<div className="linear-listings__filters">

				<div className="linear-listings__filters__row">
					<CommissionType />
				</div>

				<div className="linear-listings__filters__row">
					<SearchOptions showSearch={showSearch} />
					<ProductGroup />
				</div>

				{ showExtendedFilters && (
					<>
						<div className="linear-listings__filters__row">
							<RoomCount />
						</div>

						<div className="linear-listings__filters__row" data-group>
							<ListingType />
							<Specifications />
						</div>

						{ showBusinessListingType && (
							<div className="linear-listings__filters__row">
								<BusinessListingType />
							</div>
						)}
					</>
				)}

				{/*
					<div className="linear-listings__filters__row">
						<PriceRange />
					</div>

					<div className="linear-listings__filters__row">
						<RentRange />
					</div>
				*/}
			</div>
		);
	}

	// return with logic
	return (
		<div className="linear-listings__filters">
			{
			['APARTMENTS', 'RENT_APARTMENT'].includes(listingsType) ? (
				<>
					<div className="linear-listings__filters__row">
						<SearchOptions showSearch={showSearch} />
						<ProductGroup />
					</div>
				</>
			) : (
				''
			)}

			{['APARTMENTS', 'RENT_APARTMENT'].includes(listingsType) && showExtendedFilters ? (
				<>
					<div className="linear-listings__filters__row">
						<RoomCount />
					</div>
					<div className="linear-listings__filters__row" data-group>
						<ListingType />
						<Specifications />
					</div>
				</>
			) : (
				''
			)}

			{listingsType === 'BUSINESS_PREMISES' ? (
				<>
					<div className="linear-listings__filters__row">
						<BusinessListingType />
					</div>
				</>
			) : (
				''
			)}

			{['APARTMENTS', 'BUSINESS_PREMISES'].includes(listingsType) && showRangeSliders ? (
				<>
					<div className="linear-listings__filters__row">
						<PriceRange />
					</div>
				</>
			) : (
				''
			)}

			{listingsType === 'RENT_APARTMENT' && showRangeSliders ? (
				<>
					<div className="linear-listings__filters__row">
						<RentRange />
					</div>
				</>
			) : (
				''
			)}
		</div>
	);
};

// Wrapper around the filters to add a accordion-element
const Filters = ({showFilters, showRangeSliders, showSearch}) => {
	const { texts } = useContext(AppContext);
	const [accordionOpen, setAccordionOpen] = useState(false);
	const [windowWidth, setWindowWidth] = useState(window.innerWidth);
	const breakpoint = 768;

	useEffect(() => {
		const onResize = () => {
			setWindowWidth(window.innerWidth);
		};

		window.addEventListener('resize', onResize);

		return () => {
			window.removeEventListener('resize', onResize);
		};
	}, []);

	useEffect(() => {
		if( windowWidth < breakpoint ){
			setAccordionOpen(false);
		} else {
			setAccordionOpen(true);
		}
	}, [windowWidth]);

	if( !showFilters ){
		return '';
	}

	return (
		<div className="linear-listings__accordion">

			{ windowWidth < breakpoint && 
				<div className="wp-block-button linear-listings__accordion__toggle elementor-cta__button-wrapper">
					<button
						className="wp-block-button__link elementor-cta__button"
						onClick={() => setAccordionOpen(!accordionOpen)}
						aria-expanded={accordionOpen}
					>
						{accordionOpen ? texts.hideFilters : texts.showFilters}
					</button>
				</div>
			}

			<div className="linear-listings__accordion__content" aria-hidden={!accordionOpen} data-open={accordionOpen}>
				<AnimatePresence>
					<FiltersList showRangeSliders={showRangeSliders} showSearch={showSearch} />
				</AnimatePresence>
			</div>

		</div>
	);
};

export default Filters;
