/**
 * Filtering listings according to the frontend filters
 */
const ListingsFilterer = ({ listings, filters }) => {

	const windowConfig = window.blockLinearListingsConfig;
	const texts = windowConfig.texts;

	// commission type
	if (typeof filters['commissionType'] !== 'undefined') {

		if (filters['commissionType'] !== '') {
			listings = listings.filter((listing) => {
				if ( typeof listing['rawDataForFiltering']['commissionType'] !== 'undefined' ) {
					if ( (listing['rawDataForFiltering']['commissionType']).toLowerCase() === filters['commissionType'].toLowerCase() ) {
						return true;
					}
				}

				return false;
			});
		}
	}

	// Product group
	if (typeof filters['productGroup'] !== 'undefined') {

		if (filters['productGroup'] !== '') {
			let newlyConstructedBypass = false;

			// hotfix
			if( filters['productGroup'] === 'apartments' ) {
				newlyConstructedBypass === true;
			}

			listings = listings.filter((listing) => {
				if ( typeof listing['rawDataForFiltering']['productGroup'] !== 'undefined' ) {
					if ( (listing['rawDataForFiltering']['productGroup']).toLowerCase() === filters['productGroup'] ) {
						return true;
					}

					// hotfix
					if( newlyConstructedBypass ){
						if ( (listing['rawDataForFiltering']['productGroup']).toLowerCase() === 'newly_constructed' ) {
							return true;
						}
					}
				}

				return false;
			});
		}
	}

	// RoomCount
	if (typeof filters.roomCount !== 'undefined') {
		if( !filters.roomCount.every(count => !count) ){

			listings = listings.filter((listing) => {

				let matchingRoomCount = false;

				if (listing.roomCount !== null && typeof listing.roomCount !== 'undefined') {
					filters.roomCount.forEach((count, index) => {
						
						if (count) {
							let currentCount = parseInt(index) + 1;

							// If filtering is set to max aka. 6
							if (currentCount !== 6) {
								if (currentCount === parseInt(listing.roomCount)) {
									matchingRoomCount = true;
								}
							} else {
								// Check more equal as 6
								if (currentCount <= parseInt(listing.roomCount)) {
									matchingRoomCount = true;
								}
							}
						}
					});

					if( matchingRoomCount ){
						return true;
					}

					return false;
				}

				return false;
			});

		}

	}

	// Listing-type
	if (typeof filters['listingType'] !== 'undefined') {
		if (filters['listingType'] !== '') {
			listings = listings.filter((listing) => {
				if (
					typeof listing['rawDataForFiltering']['listingType'] !==
					'undefined'
				) {
					if (
						listing['rawDataForFiltering'][
							'listingType'
						].toLowerCase() === filters['listingType']
					) {
						return true;
					}
				}
				return false;
			});
		}
	}

	// Specifications
	['sauna', 'hasBalcony', 'housingCooperativeElevator'].forEach(
		(specification) => {
			if (typeof filters[specification] !== 'undefined') {
				if ( Array.isArray( filters[specification] ) ) {
					if( filters[specification][0] === true ){
						listings = listings.filter((listing) => {
							if ( typeof listing[specification] !== 'undefined') {
								if ( listing[specification] === true || listing[specification].toString().toLowerCase() === texts.boolTrue.toLowerCase() ) {
									return true;
								}
							}
							return false;
						});
					}
				}
			}
		}
	);

	// Price ranges
	if ( typeof filters['priceMin'] !== 'undefined' ) {
		listings = listings.filter((listing) => {
			// Handling min price
			if ( filters['priceMin'] === 0 ) {
				return true;
			}

			if ( listing['rawDebtFreePrice'] !== null ) {
				// Normal filtering
				if (listing['rawDebtFreePrice'] >= filters['priceMin']) {
					return true;
				}
			}

			return false;
		});
	}

	if ( typeof filters['priceMax'] !== 'undefined' ) {
		listings = listings.filter((listing) => {
			// Handling max price
			if ( filters['priceMax'] === 750000 ) {
				return true;
			}

			if ( listing['rawDebtFreePrice'] !== null ) {
				// Normal filtering
				if (listing['rawDebtFreePrice'] <= filters['priceMax']) {
					return true;
				}
			}

			return false;
		});
	}

	// Rent ranges
	if ( typeof filters['rentMin'] !== 'undefined' ) {
		listings = listings.filter((listing) => {
			// Handling min rent
			if ( filters['rentMin'] === 0 ) {
				return true;
			}

			if ( listing['rawRent'] !== null ) {
				// Normal filtering
				if (listing['rawRent'] >= filters['rentMin']) {
					return true;
				}
			}

			return false;
		});
	}

	if (typeof filters['rentMax'] !== 'undefined') {
		listings = listings.filter((listing) => {
			// Handling max rent
			if ( filters['rentMax'] === 1500 && listing['rawRent'] >= 1500 ) {
				return true;
			}

			if ( listing['rawRent'] !== null ) {
				// Normal filtering
				if (listing['rawRent'] <= filters['rentMax']) {
					return true;
				}
			}

			return false;
		});
	}

	// Business listing type
	if (typeof filters['businessListingType'] !== 'undefined') {
		if (filters['businessListingType'] !== '') {
			listings = listings.filter((listing) => {
				if (typeof listing['rawDataForFiltering']['listingType'] !== 'undefined') {
					if (
						listing['rawDataForFiltering']['listingType'].toLowerCase() ===
						filters['businessListingType']
					) {
						return true;
					}
				}
				return false;
			});
		}
	}

	// Search options
	if (typeof filters['searchOption'] !== 'undefined') {
		if ( filters['searchOption'] !== '') {

			const comparables = [
				'address',
				'city',
				'municipality',
				'districtFree',
			];

			listings = listings.filter((listing) => {
				return comparables.some((comparable) => {
					if( typeof listing[comparable] !== 'undefined' && listing[comparable] ){
						if( (listing[comparable]).toLowerCase().includes( filters['searchOption'].toLowerCase() ) ){
							return true;
						}
					}
				});
			});
		}
	}

	// Search
	if (typeof filters['search'] !== 'undefined') {
		if (filters['search'] !== '') {

			const comparableKeys = [
				'id',
				'identifier',
				'address',
				'city',
				'municipality',
				'districtFree'
			];

			listings = listings.filter((listing) => {

				return comparableKeys.some((comparableKey) => {

					// split search term to array
					let searchTerms = filters['search'].toLowerCase().split(' ');
					let returnValue = false;

					searchTerms.forEach(( searchTerm ) => {

						if( typeof listing[comparableKey] !== 'undefined' && listing[comparableKey] && !returnValue ){
							if( (listing[comparableKey].toString()).toLowerCase().includes( searchTerm ) ){
								returnValue = true;
							}
						}

					});

					if( returnValue ){
						return true;
					}

				});
			});
		}
	}

	return listings;
};

export default ListingsFilterer;
