<?php
/**
 * Displays plugin settings page and takes care of saving.
 * 
 * @package Linear
 */

class Linear_Settings {

	/**
	 * Core plugin class instance.
	 *
	 * @var Linear
	 */
	protected static $linear;
	protected $api_key;

	/**
	 * Defines class properties.
	 */
	public function __construct() {
		if ( is_null( self::$linear ) ) {
			self::$linear = Linear::get_instance();
		}

		$this->api_version = ( self::$linear->get_option( 'api_key' ) ? '1.1' : 'v1' );
	}
 
	/**
	 * Enqueue necessary scripts for plugin settings page.
	 */
	function admin_page_scripts() {
		if ( ! did_action( 'wp_enqueue_media' ) ) {
			wp_enqueue_media();
		}
	}

	/**
	 * Adds settings page in admin dashboard.
	 */
	public function add_plugin_page() {
		$title = 'Linear';

		if( $this->api_version === 'v1'){
			$title = $title . " <span class='update-plugins count-1'><span class='plugin-count' aria-hidden='true'>1</span><span class='screen-reader-text'>" . __('Update your API credentials', 'linear') . "</span></span>";
		}

		// Init admin page
		add_menu_page(
			'Linear settings',
			$title,
			'manage_options',
			'linear-settings',
			'',
			'dashicons-admin-multisite',
			81
		);

		// Create link to main option page
		add_submenu_page(
			'linear-settings',
			__( 'Settings', 'linear' ),
			__( 'Settings', 'linear' ),
			'manage_options',
			'linear-settings',
			array( $this, 'create_admin_page' )
		);

		// Instructions page
		add_submenu_page(
			'linear-settings',
			__( 'Instructions', 'linear' ),
			__( 'Instructions', 'linear' ),
			'manage_options',
			'linear-instructions',
			array( $this, 'create_admin_instructions' )
		);

		// Advanced usage page
		add_submenu_page(
			'linear-settings',
			__( 'Advanced usage', 'linear' ),
			__( 'Advanced usage', 'linear' ),
			'manage_options',
			'linear-advanced',
			array( $this, 'create_admin_guide' )
		);

		// Advanced usage page
		add_submenu_page(
			'linear-settings',
			__( 'Feedback', 'linear' ),
			__( 'Feedback', 'linear' ),
			'manage_options',
			'linear-feedback',
			array( $this, 'create_admin_feedback' )
		);
	}

	/**
	 * Create settings page.
	 */
	public function create_admin_page() {
		?>
		<div class="wrap">
			<h2><?php esc_html_e( 'Linear settings', 'linear' ); ?></h2>
			<p></p>
			<?php settings_errors(); ?>

			<form method="post" action="options.php">
				<?php
					settings_fields( 'option_group' );
					do_settings_sections( 'linear-settings-admin' );
					submit_button();
				?>
			</form>
		</div>
		<?php 
	}

	/**
	 * Create instructions page
	 */
	public function create_admin_instructions() { 
		?>
		<div class="wrap">
			<h1><?php echo __( 'Instructions', 'linear' ); ?></h1>

			<h2><?php echo __('What Linear plugin is for', 'linear') ?></h2>

			<p>
				<?php echo __('The Linear plugin is used to list apartments, rentals, business premises and buy-commissions on your site with ease. 
				Set your credentials and make sure you have a set listings-page and you are set to go.', 'linear'); ?>
			<p>

			<br>
			<h2><?php echo __('Prerequisites', 'linear') ?></h2>

			<p>
				<strong>
				<?php echo __('Are you already registered to Linear?', 'linear'); ?>
				</strong><br>
				<?php echo __('If not, you need to register in Linear to get the credentials for this plugin. The sales team can provide you with test-credentials for test purposes.', 'linear'); ?>
				<br><a class="button" href="https://linear.fi/"><?php echo __('Sign up to Linear', 'linear'); ?></a>

				<br><br><strong>
				<?php echo __('Permalink settings', 'linear'); ?>
				</strong><br>
				<?php echo __('Make sure your permalink settings are set to be "article name".', 'linear'); ?>

				<br><br><strong>
					<?php echo __('Listings page', 'linear'); ?>
				</strong><br>
				<?php echo __('Upon activating this plugin, we have created a listings page for you. If you want a sub-page to be the page for listings, you can change this in Linear settings.', 'linear'); ?>
			<p>

			<br>
			<h2><?php echo __('Plugin adjustments', 'linear') ?></h2>

			<p>
				<strong>
				<?php echo __('Rentals, Business premises and Buy commissions', 'linear'); ?>
				</strong><br>
				<?php echo __('These are not set automatically but can be set manually. E.g. to have a separate rentals page you need first to create a new Page (lets call it "Rentals"). 
				Once that is done, you can choose in the Linear plugin settings the page that should be used for rentals, and you choose the page you just created. ', 'linear'); ?>

				<br><br><strong>
				<?php echo __('Page width', 'linear'); ?>
				</strong><br>
				<?php echo __('If it looks like the Listing pages are too thin compared to he rest of the page, you can try changing the used template on the page with the issue. 
				Linear plugin provides a plain and wide-enough template for your use called "linear-plain". You can also use any other page-template that outputs WordPress native content.', 'linear'); ?>
			<p>

			<br>
			<h2><?php echo __('Troubleshooting', 'linear') ?></h2>
			
			<strong>
				<?php echo '<p>' . __('The listings are not visible', 'linear') . '</p>'; ?>
			</strong>
			<?php echo '<p>' . __('There are several reasons why this could happen, please check these steps in order and you should get the listings visible:', 'linear') . '</p>'; ?>
			<?php echo '
				<div class="linear__advanced__section" style="margin-top: 0; margin-bottom: 0;">
					<strong style="display: block;">' . __( 'Are the credentials right?','linear' ) . '</strong>
					<span>' . __( 'Please make sure that the API-URL and API-Key are set as instructed and that you\'ve agreed to the terms and conditions.','linear' ) . '</span>
					<br><br>
					<strong style="display: block;">' . __( 'Do you have a set listings page?','linear' ) . '</strong>
					<span>' . __( 'You need to have a set listings page to be able to get the listings visible. If your site has multiple languages, you need to set the listings page on each language that you wish the listings to work on. The listings can be included via e.g. shortcodes or gutenberg blocks, but it\'s still necessary to have a dedicated listings page for single listings.','linear' ) . '</span>
					<br><br>
					<strong style="display: block;">' . __( 'The listings-page is set but I can\'t see any listings?','linear' ) . '</strong>
					<span>' . __( 'Depending on the WordPress theme that you use, your theme might have a non-standard built page-template. The Linear WordPress plugin provides a simple page-template (linear-plain) for your use in cases like this where the theme might not support native WordPress content hooks. Please try changing the page-template of the listings page to "linear-plain".','linear' ) . '</span>
					<br>
				</div>
			'; ?>

			<br>
			<strong>
				<?php echo '<p>' . __('A single listing is not showing', 'linear') . '</p>'; ?>
			</strong>
			<?php echo '
				<div class="linear__advanced__section" style="margin-top: 0; margin-bottom: 0;">
					<strong style="display: block;">' . __( 'Re-save permalink settings','linear' ) . '</strong>
					<span>' . __( 'Each single listing has their own dynamic page, but the logic for this to work requires that the permalinks are saved. Please save the permalinks without doing any changes to the permalink settings.','linear' ) . '</span>
					<br><br>
					<strong style="display: block;">' . __( 'What permalink settings are you using?','linear' ) . '</strong>
					<span>' . __( 'Please make sure that you only use pretty-permalinks as other permalink formats might cause issues with the Linear plugin.','linear' ) . '</span>
					<br>
				</div>
			'; ?>
		
			<br>
			<strong>
				<?php echo '<p>' . __('Other issues', 'linear') . '</p>'; ?>
			</strong>
			<?php echo 'In case of other issues, please be in touch with your person at Linear and we\'ll do our best to resolve your issue.'; ?>
			
		</div>
		<?php
	}

	/**
	 * Create advanced usage page
	 */
	public function create_admin_guide() { 
		?>
		<div class="wrap">
			<h1><?php _e( 'Advanced usage', 'linear' ); ?></h1>
			
			<?php _e( 'The Linear plugin contains quite a bit of logic under the hood and provides ease of access to its features.', 'linear' ); ?>

			<ul>
				<li>
					<a href="#shortcode"><?php _e( 'Shortcodes', 'linear' ); ?></a>
					<ul style="list-style: initial; margin-left: 20px;">
						<li style="margin-bottom: 0px;"><a href="#listings"><?php _e('Listings', 'linear'); ?></a></li>
						<li style="margin-bottom: 0px;"><a href="#buy-commissions"><?php _e('Buy commissions', 'linear'); ?></a></li>
					</ul>
				</li>
				<li><a href="#restapi"><?php _e( 'REST-API', 'linear' ); ?></a></li>
			</ul>

			<?php
				// Main guides
				echo $this->shortcode_guide( "shortcode" );
				echo $this->rest_api_guide( "restapi" );
			?>

		</div>
		<?php
	}

	/**
	 * Create advin feedback
	 */
	public function create_admin_feedback() { 
		?>
		<div class="wrap">
			<h1><?php _e( 'Feedback and suggestions', 'linear' ); ?></h1>
			
			<?php echo('
				<p>' . __('If you feel like there is some feature/filter missing or something should be different, please let us know as we actively try to develop this plugin and are open for suggestions and improvements.', 'linear') . '</p>
				<p>' . __('You can share your thoughts via email to <a href="mailto:info@linear.fi">info@linear.fi</a>', 'linear') . '</p>
			'); ?>

		</div>
		<?php
	}

	/**
	 * Add and register settings sections and fields.
	 */
	public function page_init() {
		global $TRP_LANGUAGE;
		$section_key = 'setting_section';
		$languages = self::$linear->get_languages();
		$admin_language = self::$linear->get_admin_language();
		
		register_setting(
			'option_group',
			'linear_settings',
			array( $this, 'sanitize' )
		);

		add_settings_section(
			$section_key,
			__( 'Settings', 'linear' ),
			array( $this, 'section_info' ),
			'linear-settings-admin'
		);

		add_settings_field(
			'company_id',
			__( 'Company ID', 'linear' )  . ' <span style="color:red">*</span>',
			array( $this, 'company_id_callback' ),
			'linear-settings-admin',
			$section_key
		);

		add_settings_field(
			'api_url',
			__( 'API url', 'linear' ) . ' <span style="color:red">*</span>',
			array( $this, 'api_url_callback' ),
			'linear-settings-admin',
			$section_key
		);

		add_settings_field(
			'api_key',
			__( 'API key', 'linear' ) . ' <span style="color: red;">*</span>',
			array( $this, 'api_key_callback' ),
			'linear-settings-admin',
			$section_key
		);

		add_settings_field(
			'terms_of_service',
			__( 'Terms of service', 'linear' ) . ' <span style="color:red">*</span>',
			array( $this, 'terms_of_service_callback' ),
			'linear-settings-admin',
			$section_key
		);

		add_settings_field(
			'contact_url',
			__( 'Contact request URL', 'linear' ),
			array( $this, 'contact_url_callback' ),
			'linear-settings-admin',
			$section_key
		);

		add_settings_field(
			'company_logo',
			__( 'Company Logo', 'linear' ),
			array( $this, 'company_logo_callback' ),
			'linear-settings-admin',
			$section_key
		);

		add_settings_field(
			'theme_fonts',
			__( 'Use theme fonts', 'linear' ),
			array( $this, 'theme_fonts_callback' ),
			'linear-settings-admin',
			$section_key
		);
		
		add_settings_field(
			'loan_request',
			__( 'Hide loan request button', 'linear' ),
			array( $this, 'loan_request_callback' ),
			'linear-settings-admin',
			$section_key
		);

		/*
		add_settings_field(
			'leadoo_bot',
			__( 'Use Leadoo InPageBot', 'linear' ),
			array( $this, 'leadoo_bot_callback' ),
			'linear-settings-admin',
			$section_key
		);
		*/

		if( count( $languages ) > 1 ){
			foreach( $languages as $lang ){
				add_settings_field(
					'search_options_' . $lang,
					__( 'Default search options', 'linear' ) . ' - ' . $lang,
					array( $this, 'search_options_callback' ),
					'linear-settings-admin',
					$section_key,
					array( 'lang' => $lang, 'class' => 'search_options_' . $lang )
				);
			}
		} else {
			add_settings_field(
				'search_options',
				__( 'Default search options', 'linear' ),
				array( $this, 'search_options_callback' ),
				'linear-settings-admin',
				$section_key
			);
		}

		foreach ( array(
			array( 
				'id' => 'primary_color',
				'title' => __( 'Primary color', 'linear' ),
				'default_color' => '#1890ff'
			),
			array(
				'id' => 'solid_color',
				'title' => __( 'Solid color', 'linear' ),
				'default_color' => '#096dd9'
			),
			array(
				'id' => 'outline_color',
				'title' => __( 'Outline color', 'linear' ),
				'default_color' => '#0050b3'
			)
		) as $props ) {
			add_settings_field(
				$props['id'],
				$props['title'],
				array( $this, 'select_color_callback' ),
				'linear-settings-admin',
				$section_key,
				$props
			);
		}

		if( count( $languages ) > 1 ){
			foreach( $languages as $lang ){

				$required = ' <span style="color:red">*</span>';
				if( !$admin_language && isset( $TRP_LANGUAGE ) ){
					if( $TRP_LANGUAGE !== $lang ){
						$required = '';
					}
				}

				add_settings_field(
					'listings_page_' . $lang,
					__( 'Listing page', 'linear' ) . ' - ' . $lang . $required,
					array( $this, 'select_page_callback' ),
					'linear-settings-admin',
					$section_key,
					array( 'id' => 'listings_page', 'lang' => $lang, 'class' => 'listings_page_' . $lang )
				);
	
				add_settings_field(
					'rentals_page_' . $lang,
					__( 'Rentals page', 'linear' ) . ' - ' . $lang,
					array( $this, 'select_page_callback' ),
					'linear-settings-admin',
					$section_key,
					array( 'id' => 'rentals_page', 'lang' => $lang, 'class' => 'rentals_page_' . $lang )
				);
	
				add_settings_field(
					'workplace_page_' . $lang,
					__( 'Business premises page', 'linear' ) . ' - ' . $lang,
					array( $this, 'select_page_callback' ),
					'linear-settings-admin',
					$section_key,
					array( 'id' => 'workplace_page', 'lang' => $lang, 'class' => 'workplace_page_' . $lang )
				);
	
				add_settings_field(
					'buy_commissions_page_' . $lang,
					__( 'Buy commissions page', 'linear' ) . ' - ' . $lang,
					array( $this, 'select_page_callback' ),
					'linear-settings-admin',
					$section_key,
					array( 'id' => 'buy_commissions_page', 'lang' => $lang, 'class' => 'buy_commissions_page_' . $lang )
				);
			}
		} else {
			add_settings_field(
				'listings_page',
				__( 'Listing page', 'linear' ) . ' <span style="color:red">*</span>',
				array( $this, 'select_page_callback' ),
				'linear-settings-admin',
				$section_key,
				array( 'id' => 'listings_page' )
			);

			add_settings_field(
				'rentals_page',
				__( 'Rentals page', 'linear' ),
				array( $this, 'select_page_callback' ),
				'linear-settings-admin',
				$section_key,
				array( 'id' => 'rentals_page' )
			);

			add_settings_field(
				'workplace_page',
				__( 'Business premises page', 'linear' ),
				array( $this, 'select_page_callback' ),
				'linear-settings-admin',
				$section_key,
				array( 'id' => 'workplace_page' )
			);

			add_settings_field(
				'buy_commissions_page',
				__( 'Buy commissions page', 'linear' ),
				array( $this, 'select_page_callback' ),
				'linear-settings-admin',
				$section_key,
				array( 'id' => 'buy_commissions_page' )
			);
		}
	}

	/**
	 * Returns sanitized and validated settings values.
	 * 
	 * @return array
	 */
	public function sanitize( $input ) {
		$languages = self::$linear->get_languages();

		$sanitary_values = array();
		$value = null;
		foreach ( $input as $name => $value ) {
			if( count( $languages ) > 1 ){
				foreach( $languages as $lang ){
					if ( 'search_options_' . $lang === $name ) {
						// Remove unnecessary multiple spaces
						$value = preg_replace( '/\h+/', ' ', $value );
						// Remove empty lines of text and split to array of lines
						$value = preg_split( "/(^[\r\n]*|[\r\n]+)[\s\t]*[\r\n]+/", $value );
						// Sanitize text fields
						$value = array_map( 'sanitize_text_field', $value );
					}
					
					if ( in_array( $name, array( 'listings_page_' . $lang, 'rentals_page_' . $lang, 'workplace_page_' . $lang, 'buy_commissions_page_' . $lang ), true ) && is_numeric( $value ) ) {
						$value = absint( $value );
					}
				}
			}

			if ( 'search_options' === $name ) {
				// Remove unnecessary multiple spaces
				$value = preg_replace( '/\h+/', ' ', $value );
				// Remove empty lines of text and split to array of lines
				$value = preg_split( "/(^[\r\n]*|[\r\n]+)[\s\t]*[\r\n]+/", $value );
				// Sanitize text fields
				$value = array_map( 'sanitize_text_field', $value );
			}
			
			if ( 'api_url' === $name ) {
				$value = esc_url_raw( rtrim( strtok( $value, '?'), '/') );
			}
			
			if ( 'api_key' === $name ) {
				$value = sanitize_text_field( $value );
			}
			
			if ( 'contact_url' === $name ) {
				$value = esc_url_raw( $value );
			}
			
			if ( in_array( $name, array( 'outline_color', 'solid_color', 'primary_color' ), true ) ){
				$value = sanitize_hex_color( $value );
			}
			
			if ( 'theme_fonts' === $name || 'terms_of_service' === $name || 'loan_request' === $name ) {
				$value = isset( $value ) && true == $value ? 1 : 0;
			}
			
			if ( in_array( $name, array( 'listings_page', 'company_id', 'company_logo', 'rentals_page', 'workplace_page', 'buy_commissions_page' ), true ) && is_numeric( $value ) ) {
				$value = absint( $value );
			}
			
			/* if ( in_array( $name, array( 'leadoo_bot' ), true ) ) {
				$value = sanitize_text_field( $value );
			} */

			if( $value ){
				$sanitary_values[$name] = $value;
			} else {
				// Jump to next input if name is unknown and value is not sanitized.
				continue;
			}
		}

		return $sanitary_values;
	}

	public function linear_settings_pre_update_callback( $new_value, $old_value ) {
		if (! isset( $old_value['company_id'] ) || ( isset( $new_value['company_id'] ) && $new_value['company_id'] === $old_value['company_id'] ) ) {
			$company_id = self::$linear->get_option( 'company_id' );
		} else {
			$company_id = $new_value['company_id'];
			// Force refresh of leadoo_bot data if company_id changed.
			/*
			if ( isset( $old_value['leadoo_bot'] ) ) {
				unset( $old_value['leadoo_bot'] );
				$new_value['leadoo_bot'] = 1;
			}
			*/
		}

		/*
		if ( ( isset( $new_value['leadoo_bot'] ) xor isset( $old_value['leadoo_bot'] ) ) ) {
			if ( isset( $new_value['leadoo_bot'] ) ) {
				unset( $new_value['leadoo_bot'] );
				$request = self::$linear->get_option( 'api_url' ) . '/plugin/company/' . (string) $company_id;
				$request = wp_remote_get( $request );
				if ( is_wp_error( $request ) ) {
					error_log( $request->get_error_message() );
				} else {
					$request = json_decode( wp_remote_retrieve_body( $request ), true );
					if ( isset( $request['pluginLeadooUrl'] ) && ! empty( $request['pluginLeadooUrl'] ) ) {
						$new_value['leadoo_bot'] = sanitize_text_field( $request['pluginLeadooUrl'] );
					} else {
						add_settings_error( 'linear_settings', "leadoo_bot_error", esc_html( 'Leadoo InPageBot is not currently confiugred on your Linear profile.', 'linear' ), 'warning' );
					}
				}
			} else {
				unset( $new_value['leadoo_bot'] );
			}
		}
		*/

		// Each listing page must be different page, remove duplicates.
		$listing_pages = array();
		$languages = self::$linear->get_languages();

		// TranslatePress workaround, don't limit IDs
		if( in_array( 
			'translatepress-multilingual/index.php', 
			apply_filters( 'active_plugins', get_option( 'active_plugins' ) )
		) ){
			return $new_value;
		}

		if( count($languages) > 1 ){
			foreach( $languages as $lang ){
				foreach ( array( 'listings_page_' . $lang, 'rentals_page_' . $lang, 'workplace_page_' . $lang, 'buy_commissions_page_' . $lang ) as $key ) {
					if ( isset( $new_value[$key] ) && $new_value[$key] ) {
						if ( in_array( $new_value[$key], $listing_pages ) ) {
							unset( $new_value[$key] );
						} else {
							$listing_pages[] = $new_value[$key];
						}
					}
				}
			}
		} else {
			foreach ( array( 'listings_page', 'rentals_page', 'workplace_page', 'buy_commissions_page' ) as $key ) {
				if ( isset( $new_value[$key] ) && $new_value[$key] ) {
					if ( in_array( $new_value[$key], $listing_pages ) ) {
						unset( $new_value[$key] );
					} else {
						$listing_pages[] = $new_value[$key];
					}
				}
			}
		}

		return $new_value;
	}

	public function linear_settings_updated_callback( $old_value, $new_value ) {
		$flush_rewrite_rules = false;

		$languages = self::$linear->get_languages();

		// Validate if we need permalink flush
		if( count($languages) > 1 ){
			foreach( $languages as $lang ){
				foreach ( array( 'listings_page_' . $lang, 'rentals_page_' . $lang, 'workplace_page_' . $lang, 'buy_commissions_page_' . $lang ) as $key ) {
					if ( 
						( isset( $old_value[$key] ) && isset( $new_value[$key] ) && $old_value[$key] !== $new_value[$key] ) ||
						( !isset( $old_value[$key] ) && isset( $new_value[$key] ) ) ||
						( !isset( $new_value[$key] ) && isset( $old_value[$key] ) )
					) {
						$flush_rewrite_rules = true;
					}
				}
			}
		} else {
			foreach ( array( 'listings_page', 'rentals_page', 'workplace_page', 'buy_commissions_page' ) as $key ) {
				if ( isset( $old_value[$key] ) && isset( $new_value[$key] ) && $old_value[$key] !== $new_value[$key] ) {
					$flush_rewrite_rules = true;
				}
			}
		}

		if ( $flush_rewrite_rules ) {
			Linear::listings_page_rewrite_rules( false );
			Linear::buy_commissions_page_rewrite_rules( false );
			flush_rewrite_rules();
		}

		// Validate if we need to set page-templates for new set pages
		if( count($languages) > 1 ){
			foreach( $languages as $lang ){
				foreach ( array( 'listings_page_' . $lang, 'rentals_page_' . $lang, 'workplace_page_' . $lang, 'buy_commissions_page_' . $lang ) as $key ) {
					if( isset($old_value[$key]) && isset($new_value[$key]) ){
						if( $old_value[$key] !== $new_value[$key] ){
							$this->set_linear_page_template( $new_value[$key] );
						}
					} else if ( !isset($old_value[$key]) && isset($new_value[$key]) ){
						$this->set_linear_page_template( $new_value[$key] );
					}
				}
			}
		} else {
			foreach ( array( 'listings_page', 'rentals_page', 'workplace_page', 'buy_commissions_page' ) as $key ) {
				if( isset($old_value[$key]) && isset($new_value[$key]) ){
					if( $old_value[$key] !== $new_value[$key] ){
						$this->set_linear_page_template( $new_value[$key] );
					}
				} else if ( !isset($old_value[$key]) && isset($new_value[$key]) ){
					$this->set_linear_page_template( $new_value[$key] );
				}
			}
		}
	}

	/**
	 * Display info for settings section.
	 */
	public function section_info() {
		
	}

	/**
	 * Display company_id field.
	 */
	public function company_id_callback() {
		printf(
			'<input class="regular-text" type="text" name="linear_settings[company_id]" id="company_id" value="%s" required>',
			esc_attr( self::$linear->get_option( 'company_id' ) )
		);
	}

	/**
	 * Display api_url field.
	 */
	public function api_url_callback() {
		printf(
			'<input class="regular-text" type="url" name="linear_settings[api_url]" id="api_url" value="%s" required>',
			esc_attr( self::$linear->get_option( 'api_url' ) )
		);
	}

	/**
	 * Display api_key field.
	 */
	public function api_key_callback() {
		printf(
			'<input class="regular-text" type="text" name="linear_settings[api_key]" id="api_key" value="%s" required>',
			esc_attr( self::$linear->get_option( 'api_key' ) )
		);
	}

	/**
	 * Display contact_url field.
	 */
	public function contact_url_callback() {
		printf(
			'<input class="regular-text" type="url" name="linear_settings[contact_url]" id="contact_url" value="%s">',
			esc_attr( self::$linear->get_option( 'contact_url' ) )
		);
	}

	/**
	 * Display company_logo field.
	 */
	public function company_logo_callback() {
		if ( $image_url = wp_get_attachment_url( self::$linear->get_option( 'company_logo' ) ) ) :
			?>
			<button data-upload="company_logo" style="height:auto; min-height:32px; line-height:0; padding:10px" type="button" class="button media-button  select-mode-toggle-button">
				<img height="100" src="<?php echo esc_url( $image_url ); ?>" />
			</button>
			<p><a href="#" data-remove="company_logo"><?php esc_html_e( 'Remove image', 'linear' ); ?></a></p>
			<input type="hidden" name="linear_settings[company_logo]" value="<?php echo esc_attr( self::$linear->get_option( 'company_logo' ) ); ?>">
			<?php
		else :
			?>
			<button data-upload="company_logo" style="height:auto; min-height:32px; line-height:0; padding:10px" type="button" class="button media-button  select-mode-toggle-button">
				<?php esc_html_e( 'Upload image', 'linear' ); ?>
			</button>
			<p><a href="#" data-remove="company_logo" style="display:none;"><?php esc_html_e( 'Remove image', 'linear' ); ?></a></p>
			<input type="hidden" name="linear_settings[company_logo]">
			<?php
		endif;
	}

	/**
	 * Display select color fields.
	 * 
	 * @param array $args Array of arguments, must contain 'id' and 'default_color' keys with values.
	 */
	public function select_color_callback( $args ) {
		extract( $args );
		printf(
			'<input data-default-color="%2$s" type="text" name="linear_settings[%1$s]" id="%1$s" value="%3$s">',
			$id,
			$default_color,
			esc_attr( self::$linear->get_option( $id, $default_color ) )
		);
	}

	/**
	 * Display select page fields.
	 * 
	 * @param array $args Array of arguments, must contain 'id' key with value.
	 */
	public function select_page_callback( $args ) {
		global $TRP_LANGUAGE;

		extract( $args );

		$plugins = apply_filters( 'active_plugins', get_option( 'active_plugins' ) );

		$languages = self::$linear->get_languages();
		if( count( $languages ) > 1 ){
			$id = $id . '_' . $lang;
		}

		$wpml = false;
		$translatepress = false;
		if( in_array( 
			'sitepress-multilingual-cms/sitepress.php', 
			$plugins
		) ) {
			$wpml = true;
		}

		if( in_array( 
			'translatepress-multilingual/index.php',
			$plugins
		) ) {
			$translatepress = true;
		}

		// WPML pages fetch fix
		if( isset( $lang ) && $wpml ){
			do_action( 'wpml_switch_language', $lang );
		}

		ob_start();
		
			$dropdown_pages_args = [
				'selected'         	=> self::$linear->get_option( $id ),
				'name'             	=> "linear_settings[$id]",
				'show_option_none' 	=> __( 'None', 'linear' ),
				'sort_column'      	=> 'post_title'
			];

			if( isset( $lang ) ){
				$dropdown_pages_args['lang'] = $lang;
			}

			wp_dropdown_pages( $dropdown_pages_args );
		
		$renderable = ob_get_clean();

		// Reset WPML pages fetch fix
		if( isset( $lang ) && $wpml ){
			do_action( 'wpml_switch_language', null );
		}

		$admin_language = self::$linear->get_admin_language();
		$required = '';

		if( isset( $lang ) && $admin_language ){
			if( $admin_language === $lang ){
				$required = 'required';
			}
		} else {
			$required = 'required';
		}

		if( $translatepress ){
			$required = '';
			if( $lang === $TRP_LANGUAGE ){
				$required = 'required';
			}
		}

		if( str_contains($id, 'listings_page') && $required ){
			$select = '<select ' . $required . ' ';
			echo str_replace( "<select ", $select, $renderable );
		} else {
			echo $renderable;
		}
	}

	/**
	 * Display search_options field.
	 */
	public function search_options_callback( $args ) {
		extract( $args );

		$option_name = "search_options";
		$languages = self::$linear->get_languages();
		if( count( $languages ) > 1 ){
			$option_name = "search_options_" . $lang;
		}

		printf(
			'<textarea style="white-space: pre-wrap;" rows="5" name="linear_settings[' . $option_name . ']" id="' . $option_name . '">%s</textarea><p class="description" id="tagline-description">%s</p>',
			empty( self::$linear->get_option( $option_name ) ) ?  '' : esc_textarea( implode( PHP_EOL, self::$linear->get_option( $option_name ) ) ),
			__( 'Each line of text will be used as one of default search options. You can use either city names, districts or addresses, eg. "Helsinki, Espoo, Vantaa"', 'linear' )
		);
	}

	/**
	 * Display theme_fonts field.
	 */
	public function theme_fonts_callback() {
		$theme_fonts = self::$linear->get_option( 'theme_fonts' );
		printf(
			'<input type="checkbox" id="theme_fonts" name="linear_settings[theme_fonts]" value="1" %s /> %s',
			checked( '1', $theme_fonts, false ),
			__( 'Check to use current WordPress theme fonts instead of Linear plugin fonts.', 'linear' )
		);
	}

	/**
	 * Display loan_request field.
	 */
	public function loan_request_callback() {
		printf(
			'<input type="checkbox" id="loan_request" name="linear_settings[loan_request]" value="1" %s />',
			checked( '1', self::$linear->get_option( 'loan_request' ), false )
		);
	}

	/**
	 * Display leadoo_bot field.
	 */
	/*
	public function leadoo_bot_callback() {
		printf(
			'<input type="checkbox" id="leadoo_bot" name="linear_settings[leadoo_bot]" value="%s" %s /> %s',
			self::$linear->get_option( 'leadoo_bot', 1 ),
			checked( true, (bool) self::$linear->get_option( 'leadoo_bot' ), false ),
			__( 'Check this to use Linear-provided Leadoo robot, if available.', 'linear' )
		);
	}
	*/

	/**
	 * Display terms_of_service field.
	 */
	public function terms_of_service_callback() {
		printf(
			'<input type="checkbox" id="terms_of_service" name="linear_settings[terms_of_service]" required value="1" %s /> %s <a target="_blank" href="%s">%s<a>.',
			checked( '1', self::$linear->get_option( 'terms_of_service' ), false ),
			__( 'Before using Linear service you need to accept our', 'linear' ),
			esc_url( 'https://linear.fi/wp-content/uploads/2022/07/Linear-Yleiset-Toimitusehdot.pdf' ),
			__( 'terms of service', 'linear' )
		);
	}

	/**
	 * Display shortcode usage
	 */
	public function shortcode_guide( $anchor_id = '' ) {
		echo('
			<div class="linear__advanced__section" id="' . $anchor_id . '">
				<h2>' . __( 'Shortcodes', 'linear' ) . '</h2>
				<div>' .
				__('You can add listings and buy commissions according to your own liking wherever you want on the site with the help of shortcodes.', 'linear') . 
				' ' . __('This is handy for sites which are not using the Gutenberg editor as the various listings can be also utilized with Shortcodes.', 'linear') . 
				' ' . __('The shortcodes support some additional settings which affect how the listing is displayed.', 'linear') . '</div>
				<br>
				
				<div class="linear__advanced__shortcode__section">
					<h2 id="listings">' . __( 'Listings', 'linear' ) . '</h2>
					<p><strong>' . __('Simple listing example:', 'linear') . '</strong></p>
					<pre><code class="language-html">
[linear_block_listings]
					</code></pre>
				</div>

				<div class="linear__advanced__shortcode__section">
					<p><strong>' . __('Advanced listing example:', 'linear') . '</strong></p>
					<pre><code class="language-html">
[linear_block_listings type="apartments" per_page="4" loadmore="false" classes="text-color-brand" filters="false" range_sliders="false" order_by="true"]
					</code></pre>
					<br><br>
					<p><strong>' . __('Supported arguments:', 'linear') . '</strong></p>
					<ol>
						<li>
							<p><strong>' . __('Type', 'linear') . ' (type)</strong></p>
							<p>' . __('The type indicates if we want to show apartment, rentals or business premises. Default is "all"', 'linear') . '</p>
							<p>' . __('Acceptable values:', 'linear') . ' <code>"all", "apartments", "rent_apartments", "business_premises"</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_listings type="all"]
								</code></pre>
							</p>
						</li>

						<li>
							<p><strong>' . __('Per page', 'linear') . ' (per_page)</strong></p>
							<p>' . __('The per page indicates how many listings we want to show, and how many more are shown upon "Load more"-button click. Default is "8"', 'linear') . '</p>
							<p>' . __('Accpeting positive numeric values:', 'linear') . ' <code>' . __('*Any positive number from 1-999*', 'linear') . '</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_listings per_page="8"]
								</code></pre>
							</p>
						</li>

						<li>
							<p><strong>' . __('Load more', 'linear') . ' (loadmore)</strong></p>
							<p>' . __('The options to enable/disable the "Load more"-button. Default is "true"', 'linear') . '</p>
							<p>' . __('Acceptable values:', 'linear') . ' <code>"true", "false"</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_listings loadmore="true"]
								</code></pre>
							</p>
						</li>

						<li>
							<p><strong>' . __('Classes', 'linear') . ' (classes)</strong></p>
							<p>' . __('Add one or several classes to add to the block. Default is ""', 'linear') . '</p>
							<p>' . __('Accepting string value:', 'linear') . ' - ' . __('eg.', 'linear') . ' <code>"class_name_one class_name_two"</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_listings classes="my_extra_class my_other_extra_class"]
								</code></pre>
							</p>
						</li>

						<li>
							<p><strong>' . __('Filters', 'linear') . ' (filters)</strong></p>
							<p>' . __('You can chose if you want to show or hide the filters for the listing. Default is "true"', 'linear') . '</p>
							<p>' . __('Acceptable values:', 'linear') . ' <code>"true", "false"</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_listings filters="true"]
								</code></pre>
							</p>
						</li>

						<li>
							<p><strong>' . __('Range sliders', 'linear') . ' (range_sliders)</strong></p>
							<p>' . __('Some filterings support range sliders for eg. price or rent. You can decide if you want to show these or not. Requires that filters are used! Default is "true"', 'linear') . '</p>
							<p>' . __('Acceptable values:', 'linear') . ' <code>"true", "false"</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_listings range_sliders="true"]
								</code></pre>
							</p>
						</li>

						<li>
							<p><strong>' . __('Show search field', 'linear') . '</strong></p>
							<p>' . __('You can decide if you want to hide or show the search-field. Default is "true" meaning it is shown.', 'linear') . '</p>
							<p>' . __('Acceptable values:', 'linear') . ' <code>"true", "false"</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_listings show_search="true"]
								</code></pre>
							</p>
						</li>

						<li>
							<p><strong>' . __('Search field value', 'linear') . '</strong></p>
							<p>' . __('You can pre-set the search field value. This is handy if you want to limit the listings shown on a page with a keyword e.g. a city name or area. Default is empty aka. ""', 'linear') . '</p>
							<p>' . __('Acceptable values:', 'linear') . ' <code>' . __('*Any searchterms you want to use separate with a space*', 'linear') . '</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_listings filter_search="helsinki"]
								</code></pre>
							</p>
						</li>
					</ol>
				</div>

				<br><br>
				<div class="linear__advanced__shortcode__section">
					<h2 id="buy-commissions">' . __( 'Buy commissions', 'linear' ) . '</h2>
					<p><strong>' . __('Simple buy commissions example:', 'linear') . '</strong></p>
					<pre><code class="language-html">
[linear_block_buy_commissions]
					</code></pre>
				</div>

				<div class="linear__advanced__shortcode__section">
					<p><strong>' . __('Advanced buy commissions example:', 'linear') . '</strong></p>
					<pre><code class="language-html">
[linear_block_buy_commissions per_page="4" loadmore="false" classes="text-color-brand" filters="false" range_sliders="true" order_by="true"]
					</code></pre>
					<br><br>
					<p><strong>' . __('Supported arguments:', 'linear') . '</strong></p>
					<ol>

						<li>
							<p><strong>' . __('Per page', 'linear') . ' (per_page)</strong></p>
							<p>' . __('The per page indicates how many buy commissions we want to show, and how many more are shown upon "Load more"-button click. Default is "8"', 'linear') . '</p>
							<p>' . __('Accpeting positive numeric values:', 'linear') . ' <code>' . __('*Any positive number from 1-999*', 'linear') . '</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_buy_commissions per_page="8"]
								</code></pre>
							</p>
						</li>

						<li>
							<p><strong>' . __('Load more', 'linear') . ' (loadmore)</strong></p>
							<p>' . __('The options to enable/disable the "Load more"-button. Default is "true"', 'linear') . '</p>
							<p>' . __('Acceptable values:', 'linear') . ' <code>"true", "false"</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_buy_commissions loadmore="true"]
								</code></pre>
							</p>
						</li>

						<li>
							<p><strong>' . __('Classes', 'linear') . ' (classes)</strong></p>
							<p>' . __('Add one or several classes to add to the block. Default is ""', 'linear') . '</p>
							<p>' . __('Accepting string value:', 'linear') . ' - ' . __('eg.', 'linear') . ' <code>"class_name_one class_name_two"</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_buy_commissions classes="my_extra_class my_other_extra_class"]
								</code></pre>
							</p>
						</li>

						<li>
							<p><strong>' . __('Filters', 'linear') . ' (filters)</strong></p>
							<p>' . __('You can chose if you want to show or hide the filters for the buy commissions. Default is "true"', 'linear') . '</p>
							<p>' . __('Acceptable values:', 'linear') . ' <code>"true", "false"</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_buy_commissions filters="true"]
								</code></pre>
							</p>
						</li>

						<li>
							<p><strong>' . __('Range sliders', 'linear') . ' (range_sliders)</strong></p>
							<p>' . __('Some filterings support range sliders for eg. price. You can decide if you want to show these or not. Requires that filters are used! Default is "true"', 'linear') . '</p>
							<p>' . __('Acceptable values:', 'linear') . ' <code>"true", "false"</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_buy_commissions range_sliders="true"]
								</code></pre>
							</p>
						</li>

						<li>
							<p><strong>' . __('Show search field', 'linear') . '</strong></p>
							<p>' . __('You can decide if you want to hide or show the search-field. Default is "true" meaning it is shown.', 'linear') . '</p>
							<p>' . __('Acceptable values:', 'linear') . ' <code>"true", "false"</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_buy_commissions show_search="true"]
								</code></pre>
							</p>
						</li>

						<li>
							<p><strong>' . __('Search field value', 'linear') . '</strong></p>
							<p>' . __('You can pre-set the search field value. This is handy if you want to limit the listings shown on a page with a keyword e.g. a city name or area. Default is empty aka. ""', 'linear') . '</p>
							<p>' . __('Acceptable values:', 'linear') . ' <code>' . __('*Any searchterms you want to use separate with a space*', 'linear') . '</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_buy_commissions filter_search="helsinki"]
								</code></pre>
							</p>
						</li>
					</ol>
				</div>
			</div>
		');
	}

	/**
	 * Display rest API guide
	 */
	public function rest_api_guide( $anchor_id = '' ) {
		$version = 'v' . str_replace( '.', '_', $this->api_version );
		$rest_endpoint = get_home_url() . '/wp-json/linear/' . $version . '/';
		$locale = substr( get_locale(), 0, 2 );

		echo('
			<div class="linear__advanced__section" id="' . $anchor_id . '">
				<h2>' . __('REST-API usage:', 'linear') . '</h2>
				<div>' . __('The Linear plugin exposes a few rest endpoints which allow you to fetch listings. The plugin handles everything with fetching the data and saving it in a transient so that you don\'t have to worry about that.', 'linear') . '</div>
				
				<h3>' . __('Listings:', 'linear') . '</h3>

				<div>
					<p><strong>' . __('Endpoints:', 'linear') . ' <a href="' . $rest_endpoint . '">' . $rest_endpoint . '</a></strong></p>
					<pre><code class="language-json">
{
    "namespace": "linear/' . $version . '",
    "routes": {
        "/linear/' . $version . '": {},
        "/linear/' . $version . '/listings/all": {},
        "/linear/' . $version . '/listings/apartments": {},
        "/linear/' . $version . '/listings/rentals": {},
        "/linear/' . $version . '/listings/business_premises": {},
        "/linear/' . $version . '/listing/SINGLE_LISTING_ID": {}
    }
}
					</code></pre>
				</div>

				<div>' . __('If you want you can also access the same data via WordPress filters.', 'linear') . '</div>
				
				<div>
					<p><strong>' . __('Available filters:', 'linear') . '</strong></p>
					<pre><code class="language-php">
$lang = null; // ' . __( 'Can be 2-letter language code or null, null returning site locale', 'linear' ) . '

$type = "rentals"; // ' . sprintf( __( 'Can be "%s", "%s" or "%s"', 'linear' ), 'apartments', 'rentals', 'buy_commissions' ) . '

apply_filters( "linear_listings", $lang );
apply_filters( "linear_listings_by_type", $type, $lang );
apply_filters( "linear_listing", "SINGLE_LISTING_ID" );
					</code></pre>
				</div>

				<div>
					<p>' . __( 'And you can use them straight like this', 'linear' ) . ':</p>
					<pre><code class="language-php">
$listings = apply_filters( "linear_listings", "fi" );

foreach( $listings as $listing ){
    echo $listing["address"];
}
					</code></pre>
				</div>
				<br><br>

				<h3>' . __('Buy commissions:', 'linear') . '</h3>

				<div>
					<pre><code class="language-json">
{
    "namespace": "linear/' . $version . '",
    "routes": {
        "/linear/' . $version . '": {},
        "/linear/' . $version . '/buy-commissions/all": {},
        "/linear/' . $version . '/buy-commission/SINGLE_BUY_COMMISSION_ID": {}
    }
}
					</code></pre>
				</div>

				<div>' . __('You can also access the same data via WordPress filters.', 'linear') . '</div>
				
				<div>
					<p><strong>' . __('Available filters:', 'linear') . '</strong></p>
					<pre><code class="language-php">
apply_filters( "linear_buy_commissions_all", null )
apply_filters( "linear_buy_commission", "SINGLE_BUY_COMMISSION_ID" )
					</code></pre>
				</div>

				<div>
					<p>' . __( 'And you can use them straight like this', 'linear' ) . ':</p>
					<pre><code class="language-php">
$buy_commissions = apply_filters( "linear_buy_commissions_all", null );

foreach( $buy_commissions as $buy_commission ){
    echo $buy_commission["data"]["location"]["' . $locale . '"]["value"];
}
					</code></pre>
				</div>
			</div>
		');
	}

	public function admin_body_class( $classes ){

		$admin_language = self::$linear->get_admin_language();

		if( !$admin_language ){

			// translatepress workaround
			if( $languages = self::$linear->get_languages() ){
				$additional_classes = '';

				foreach( $languages as $lang ){
					$additional_classes .= ' linear-' . $lang . ' ';
				}

				return $classes . ' - ' . $additional_classes;
			}

			// fallback
			return $classes;
		}

		$classes = $classes . ' linear-' . $admin_language . ' ';

		return $classes;
	}

	/**
	 * Sets page-template for $post_id
	 */
	private function set_linear_page_template( $post_id ){
		if( !$post_id || !intval($post_id) ){
			return;
		}

		update_post_meta( intval($post_id), '_wp_page_template', 'plain-listings.php' );
	}
}
