<?php

namespace Linear\Templates;

function realtor( $listing = null ) {

    if( !$listing ){
        return '';
    }

    $realtor = $listing['realtor'];

    if ( !$realtor ) {
        return '';
    }

    $use_realtor_data = false;
    if( isset( $listing['contactInfoSource'] ) && $listing['contactInfoSource'] === 'REALTOR' ){
        $use_realtor_data = true;
    }

    $contact_link = '';
    if( esc_html( get_linear_option_value('contact_url') ) ){
        $contact_link = esc_html( get_linear_option_value('contact_url') );

        $contact_link_query = parse_url( esc_html( get_linear_option_value('contact_url') ), PHP_URL_QUERY );

        $additive_listing_id = 'listing_id=' . $listing['id'];
        $additive_listing_name = 'listing_address=' . urlencode( $listing['address'] );

        if( $contact_link_query ){
            $contact_link .= '&';
        } else {
            $contact_link .= '?';
        }

        $contact_link .= $additive_listing_id . '&' . strtolower( $additive_listing_name );

    } else {
        $contact_link = esc_html( get_listing_link( $listing, 'dixu_listing_url' ) );
    }

    ob_start(); ?>

        <div class="linear-o-row">
            <div class="linear-o-col-12">
                <div class="linear-c-card">
                    <div class="linear-o-row linear-u-items-center linear-u-text-center linear-u-text-left@md">
                        <div class="linear-o-col-5 linear-o-col-2@md linear-o-col-1@xl linear-u-mx-auto">
                            <?php
                            if( $use_realtor_data ){
                                if ( isset( $realtor['avatar'] ) && $realtor['avatar'] ) { ?>
                                    <img src="<?php echo $realtor['avatar']; ?>" class="linear-c-avatar--agent linear-u-mb-2 linear-u-mb-0@md linear-u-ml-0@md linear-u-mx-auto">
                                <?php } else { ?>
                                    <div class="linear-o-icon linear-c-card__icon">
                                        <?php echo icon( 'avatar' ); ?>
                                    </div>
                                <?php }
                            } else { ?>
                                <img src="<?php echo $realtor['companyLogo']; ?>" class="linear-c-avatar--agent linear-u-mb-2 linear-u-mb-0@md linear-u-ml-0@md linear-u-mx-auto">
                            <?php } ?>
                        </div>

                        <div class="linear-o-col-7@md linear-o-col-9@xl">
                            <p class="linear-c-card__heading linear-c-card__heading--sm linear-u-mb-0.5">

                                <?php
                                    if( isset( $listing['rawDataForFiltering']['productGroup'] ) ){
                                        switch ( $listing['rawDataForFiltering']['productGroup'] ) {
                                            case 'RENT_APARTMENT':
                                                echo esc_html( __('Rent Apartment', 'linear') );
                                                break;
                                            case 'BUSINESS_PREMISES':
                                                echo esc_html( __( 'This business premise is sold by', 'linear' ) );
                                                break;
                                            default:
                                                echo esc_html( __( 'This apartment is sold by', 'linear' ) );
                                                break;
                                        }

                                        if( $use_realtor_data ){
                                            echo esc_html(
                                                sprintf(
                                                    ' %s%s',
                                                    $realtor['name'],
                                                    isset( $realtor['jobTitle'] ) && $realtor['jobTitle'] ? ', ' . $realtor['jobTitle'] : ''
                                                )
                                            );
                                        } else {
                                            echo esc_html(
                                                sprintf(
                                                    ' %s',
                                                    $realtor['companyName']
                                                )
                                            );
                                        }
                                    }
                                ?>
                            </p>

                            <?php if( !comparator_is_sub_value( $listing, ['realtor', 'companyName'], null) && $use_realtor_data ){ ?>
                                <p class="linear-u-mb-0.5"><?php echo $realtor['companyName']; ?></p>
                            <?php } else if( !comparator_is_sub_value( $listing, ['realtor', 'companyTel'], null) ){ ?>
                                <a class="linear-c-card__heading__phone" href="tel:<?php echo $listing['realtor']['companyTel']?>">
                                    <span class="linear-c-card__heading__icon"><?php echo icon( 'phone' ); ?></span>
                                    <span class="linear-c-card__heading__number"><?php echo $listing['realtor']['companyTel']?></span>
                                </a>
                            <?php } ?>

                        </div>


                        <div class="linear-o-col-3@md linear-o-col-2@xl linear-u-text-right@md">
                            <?php if ( get_constant( $listing, 'status') === 'LIGHT_MIGRATED' || get_linear_option_value('contact_url') !== null ) { ?>
                                <a 
                                    href="<?php echo $contact_link; ?>" 
                                    target="_blank" 
                                    class="linear-c-button linear-c-button--outline linear-c-button--primary linear-c-button--full"
                                >
                                    <?php esc_attr_e( 'Contact', 'linear' ); ?>
                                </a>
                            <?php } ?>

                            <div class="linear-u-mt-0.5 linear-c-avatar--company">
                                <img src="<?php echo get_company_logo( $realtor ); ?>" class="linear-c-avatar linear-c-avatar--company__content linear-u-mx-auto linear-u-ml-0@md">
                            </div>
                        </div>

                    </div>
                </div>
            </div>
        </div>
        
    <?php return ob_get_clean();
}
